@@ -17,6 +17,7 @@ import data_algebra.data_ops_utils
 import data_algebra.near_sql
 import data_algebra.OrderedSet
 
+
 _have_black = False
 try:
     # noinspection PyUnresolvedReferences
@@ -26,14 +27,19 @@ try:
 except ImportError:
     pass
 
-_have_sqlparse = False
-try:
-    # noinspection PyUnresolvedReferences
-    import sqlparse
 
-    _have_sqlparse = True
-except ImportError:
-    pass
+# noinspection PyBroadException
+def pretty_format_python(python_str, *, black_mode=None):
+    assert isinstance(python_str, str)
+    formatted_python = python_str
+    if _have_black:
+        try:
+            if black_mode is None:
+                black_mode = black.FileMode()
+            formatted_python = black.format_str(python_str, mode=black_mode)
+        except Exception:
+            pass
+    return formatted_python
 
 
 class ViewRepresentation(OperatorPlatform, ABC):
@@ -41,7 +47,7 @@ class ViewRepresentation(OperatorPlatform, ABC):
        Abstract base class."""
 
     column_names: List[str]
-    column_set: Set[str]
+    column_set: data_algebra.OrderedSet.OrderedSet
     column_map: collections.OrderedDict
     sources: List[
         "ViewRepresentation"
@@ -176,13 +182,7 @@ class ViewRepresentation(OperatorPlatform, ABC):
             indent=indent, strict=strict, print_sources=True
         )
         if pretty:
-            if _have_black:
-                try:
-                    if black_mode is None:
-                        black_mode = black.FileMode()
-                    python_str = black.format_str(python_str, mode=black_mode)
-                except Exception:
-                    pass
+            python_str = pretty_format_python(python_str, black_mode=black_mode)
         return python_str
 
     def __repr__(self):
@@ -223,47 +223,32 @@ class ViewRepresentation(OperatorPlatform, ABC):
 
     # query generation
 
-    def to_sql_implementation(self, db_model, *, using, temp_id_source):
+    def to_near_sql_implementation(self, db_model, *, using, temp_id_source):
         raise NotImplementedError("base method called")
 
-    # noinspection PyBroadException
     def to_sql(
         self,
         db_model,
         *,
         pretty=False,
+        annotate=False,
         encoding=None,
         sqlparse_options=None,
-        temp_tables=None
+        temp_tables=None,
+        use_with=False
     ):
-        if sqlparse_options is None:
-            sqlparse_options = {"reindent": True, "keyword_case": "upper"}
         if isinstance(db_model, data_algebra.db_model.DBHandle):
             db_model = db_model.db_model
-        if not isinstance(db_model, data_algebra.db_model.DBModel):
-            raise TypeError(
-                "Expected db_model to be derived from data_algebra.db_model.DBModel or data_algebra.db_model.DBHandle"
-            )
-        self.columns_used()  # for table consistency check/raise
-        temp_id_source = [0]
-        sub_sql = self.to_sql_implementation(
-            db_model=db_model, using=None, temp_id_source=temp_id_source
+        assert isinstance(db_model, data_algebra.db_model.DBModel)
+        return db_model.to_sql(
+            ops=self,
+            pretty=pretty,
+            annotate=annotate,
+            encoding=encoding,
+            sqlparse_options=sqlparse_options,
+            temp_tables=temp_tables,
+            use_with=use_with
         )
-        if (sub_sql.temp_tables is not None) and (len(sub_sql.temp_tables) > 0):
-            if temp_tables is None:
-                raise ValueError(
-                    "need temp_tables to be a dictionary to copy back found temporary table values"
-                )
-            temp_tables.update(sub_sql.temp_tables)
-        sql_str = sub_sql.to_sql(db_model=db_model, force_sql=True)
-        if pretty and _have_sqlparse:
-            try:
-                sql_str = sqlparse.format(
-                    sql_str, encoding=encoding, **sqlparse_options
-                )
-            except Exception:
-                pass
-        return sql_str
 
     # Pandas realization
 
@@ -519,12 +504,11 @@ class ViewRepresentation(OperatorPlatform, ABC):
         ops = data_algebra.expr_parse.parse_assignments_in_context(
             {"expr": expr}, self, parse_env=parse_env
         )
-        data_model = data_algebra.default_data_model
 
-        def r_walk_expr(op):
-            if not isinstance(op, data_algebra.expr_rep.Expression):
+        def r_walk_expr(opv):
+            if not isinstance(opv, data_algebra.expr_rep.Expression):
                 return
-            for oi in op.args:
+            for oi in opv.args:
                 r_walk_expr(oi)
 
         for op in ops.values():
@@ -680,7 +664,9 @@ class TableDescription(ViewRepresentation):
         return True
 
     def to_python_implementation(self, *, indent=0, strict=True, print_sources=True):
-        spacer = "\n " + " " * indent
+        spacer = ' '
+        if indent >= 0:
+            spacer = "\n " + " " * indent
         column_limit = 20
         truncated = (not strict) and (column_limit < len(self.column_names))
         if truncated:
@@ -690,7 +676,7 @@ class TableDescription(ViewRepresentation):
         else:
             cols_to_print = [c.__repr__() for c in self.column_names]
         col_text = data_algebra.flow_text.flow_text(
-            cols_to_print, align_right=70 - indent, sep_width=2
+            cols_to_print, align_right=70 - max(0, indent), sep_width=2
         )
         col_text = [", ".join(line) for line in col_text]
         col_text = (",  " + spacer).join(col_text)
@@ -727,7 +713,7 @@ class TableDescription(ViewRepresentation):
     def columns_used_from_sources(self, using=None):
         return []  # no inputs to table description
 
-    def to_sql_implementation(self, db_model, *, using, temp_id_source):
+    def to_near_sql_implementation(self, db_model, *, using, temp_id_source):
         return db_model.table_def_to_sql(
             self, using=using, temp_id_source=temp_id_source
         )
@@ -940,7 +926,7 @@ class ExtendNode(ViewRepresentation):
                         raise ValueError("window function with more than one argument")
                     for i in range(len(opk.args)):
                         if not (isinstance(opk.args[0], data_algebra.expr_rep.ColumnReference)
-                            or isinstance(opk.args[0], data_algebra.expr_rep.Value)):
+                                or isinstance(opk.args[0], data_algebra.expr_rep.Value)):
                             raise ValueError(
                                 "window expression argument must be a column or value: "
                                 + str(k)
@@ -962,7 +948,9 @@ class ExtendNode(ViewRepresentation):
         return [columns_we_take]
 
     def to_python_implementation(self, *, indent=0, strict=True, print_sources=True):
-        spacer = "\n   " + " " * indent
+        spacer = ' '
+        if indent >= 0:
+            spacer = "\n   " + " " * indent
         s = ""
         if print_sources:
             s = (
@@ -988,7 +976,7 @@ class ExtendNode(ViewRepresentation):
         s = s + ")"
         return s
 
-    def to_sql_implementation(self, db_model, *, using, temp_id_source):
+    def to_near_sql_implementation(self, db_model, *, using, temp_id_source):
         return db_model.extend_to_sql(self, using=using, temp_id_source=temp_id_source)
 
     def eval_implementation(self, *, data_map, data_model, narrow):
@@ -1040,7 +1028,7 @@ class ProjectNode(ViewRepresentation):
                     )
                 if len(opk.args) > 0:
                     if not (isinstance(opk.args[0], data_algebra.expr_rep.ColumnReference)
-                        or isinstance(opk.args[0], data_algebra.expr_rep.Value)):
+                            or isinstance(opk.args[0], data_algebra.expr_rep.Value)):
                         raise ValueError(
                             "windows expression argument must be a column or value: "
                             + str(k)
@@ -1100,13 +1088,15 @@ class ProjectNode(ViewRepresentation):
         return [columns_we_take]
 
     def to_python_implementation(self, *, indent=0, strict=True, print_sources=True):
-        spacer = "\n   " + " " * indent
+        spacer = ' '
+        if indent >= 0:
+            spacer = "\n   " + " " * indent
         s = ""
         if print_sources:
             s = (
                 self.sources[0].to_python_implementation(indent=indent, strict=strict)
                 + " .\\\n"
-                + " " * (indent + 3)
+                + " " * (max(indent, 0) + 3)
             )
         s = s + (
             "project({"
@@ -1125,7 +1115,7 @@ class ProjectNode(ViewRepresentation):
         s = s + ")"
         return s
 
-    def to_sql_implementation(self, db_model, *, using, temp_id_source):
+    def to_near_sql_implementation(self, db_model, *, using, temp_id_source):
         return db_model.project_to_sql(self, using=using, temp_id_source=temp_id_source)
 
     def eval_implementation(self, *, data_map, data_model, narrow):
@@ -1185,12 +1175,12 @@ class SelectRowsNode(ViewRepresentation):
             s = (
                 self.sources[0].to_python_implementation(indent=indent, strict=strict)
                 + " .\\\n"
-                + " " * (indent + 3)
+                + " " * (max(indent, 0) + 3)
             )
         s = s + ("select_rows(" + self.expr.to_python().__repr__() + ")")
         return s
 
-    def to_sql_implementation(self, db_model, *, using, temp_id_source):
+    def to_near_sql_implementation(self, db_model, *, using, temp_id_source):
         return db_model.select_rows_to_sql(
             self, using=using, temp_id_source=temp_id_source
         )
@@ -1256,12 +1246,12 @@ class SelectColumnsNode(ViewRepresentation):
             s = (
                 self.sources[0].to_python_implementation(indent=indent, strict=strict)
                 + " .\\\n"
-                + " " * (indent + 3)
+                + " " * (max(indent, 0) + 3)
             )
         s = s + ("select_columns(" + self.column_selection.__repr__() + ")")
         return s
 
-    def to_sql_implementation(self, db_model, *, using, temp_id_source):
+    def to_near_sql_implementation(self, db_model, *, using, temp_id_source):
         return db_model.select_columns_to_sql(
             self, using=using, temp_id_source=temp_id_source
         )
@@ -1327,12 +1317,12 @@ class DropColumnsNode(ViewRepresentation):
             s = (
                 self.sources[0].to_python_implementation(indent=indent, strict=strict)
                 + " .\\\n"
-                + " " * (indent + 3)
+                + " " * (max(indent, 0) + 3)
             )
         s = s + ("drop_columns(" + self.column_deletions.__repr__() + ")")
         return s
 
-    def to_sql_implementation(self, db_model, *, using, temp_id_source):
+    def to_near_sql_implementation(self, db_model, *, using, temp_id_source):
         return db_model.drop_columns_to_sql(
             self, using=using, temp_id_source=temp_id_source
         )
@@ -1405,7 +1395,7 @@ class OrderRowsNode(ViewRepresentation):
             s = (
                 self.sources[0].to_python_implementation(indent=indent, strict=strict)
                 + " .\\\n"
-                + " " * (indent + 3)
+                + " " * (max(indent, 0) + 3)
             )
         s = s + ("order_rows(" + self.order_columns.__repr__())
         if len(self.reverse) > 0:
@@ -1415,7 +1405,7 @@ class OrderRowsNode(ViewRepresentation):
         s = s + ")"
         return s
 
-    def to_sql_implementation(self, db_model, *, using, temp_id_source):
+    def to_near_sql_implementation(self, db_model, *, using, temp_id_source):
         return db_model.order_to_sql(self, using=using, temp_id_source=temp_id_source)
 
     def eval_implementation(self, *, data_map, data_model, narrow):
@@ -1505,12 +1495,12 @@ class RenameColumnsNode(ViewRepresentation):
             s = (
                 self.sources[0].to_python_implementation(indent=indent, strict=strict)
                 + " .\\\n"
-                + " " * (indent + 3)
+                + " " * (max(indent, 0) + 3)
             )
         s = s + ("rename_columns(" + self.column_remapping.__repr__() + ")")
         return s
 
-    def to_sql_implementation(self, db_model, *, using, temp_id_source):
+    def to_near_sql_implementation(self, db_model, *, using, temp_id_source):
         return db_model.rename_to_sql(self, using=using, temp_id_source=temp_id_source)
 
     def eval_implementation(self, *, data_map, data_model, narrow):
@@ -1595,16 +1585,16 @@ class NaturalJoinNode(ViewRepresentation):
             s = (
                 self.sources[0].to_python_implementation(indent=indent, strict=strict)
                 + " .\\\n"
-                + " " * (indent + 3)
+                + " " * (max(indent, 0) + 3)
             )
         s = s + ("natural_join(b=\n" + " " * (indent + 6))
         if print_sources:
             s = s + (
                 self.sources[1].to_python_implementation(
-                    indent=indent + 6, strict=strict
+                    indent=max(indent, 0) + 6, strict=strict
                 )
                 + ",\n"
-                + " " * (indent + 6)
+                + " " * (max(indent, 0) + 6)
             )
         else:
             s = s + " _1, "
@@ -1613,7 +1603,7 @@ class NaturalJoinNode(ViewRepresentation):
         )
         return s
 
-    def to_sql_implementation(self, db_model, *, using, temp_id_source):
+    def to_near_sql_implementation(self, db_model, *, using, temp_id_source):
         return db_model.natural_join_to_sql(
             self, using=using, temp_id_source=temp_id_source
         )
@@ -1689,16 +1679,16 @@ class ConcatRowsNode(ViewRepresentation):
             s = (
                 self.sources[0].to_python_implementation(indent=indent, strict=strict)
                 + " .\\\n"
-                + " " * (indent + 3)
+                + " " * (max(indent, 0) + 3)
             )
         s = s + ("concat_rows(b=\n" + " " * (indent + 6))
         if print_sources:
             s = s + (
                 self.sources[1].to_python_implementation(
-                    indent=indent + 6, strict=strict
+                    indent=max(indent, 0) + 6, strict=strict
                 )
                 + ",\n"
-                + " " * (indent + 6)
+                + " " * (max(indent, 0) + 6)
             )
         else:
             s = s + " _1, "
@@ -1713,7 +1703,7 @@ class ConcatRowsNode(ViewRepresentation):
         )
         return s
 
-    def to_sql_implementation(self, db_model, *, using, temp_id_source):
+    def to_near_sql_implementation(self, db_model, *, using, temp_id_source):
         return db_model.concat_rows_to_sql(
             self, using=using, temp_id_source=temp_id_source
         )
@@ -1775,7 +1765,7 @@ class ConvertRecordsNode(ViewRepresentation):
             s = (
                 self.sources[0].to_python_implementation(indent=indent, strict=strict)
                 + " .\\\n"
-                + " " * (indent + 3)
+                + " " * (max(indent, 0) + 3)
             )
         rm_str = self.record_map.__repr__()
         rm_str = re.sub("\n", "\n   ", rm_str)
@@ -1783,11 +1773,11 @@ class ConvertRecordsNode(ViewRepresentation):
         s = s + ")"
         return s
 
-    def to_sql_implementation(self, db_model, *, using, temp_id_source):
+    def to_near_sql_implementation(self, db_model, *, using, temp_id_source):
         if temp_id_source is None:
             temp_id_source = [0]
         # TODO: narrow to what we are using
-        sub_query = self.sources[0].to_sql_implementation(
+        sub_query = self.sources[0].to_near_sql_implementation(
             db_model=db_model, using=None, temp_id_source=temp_id_source
         )
         # claims to use all columns
@@ -1821,9 +1811,10 @@ class ConvertRecordsNode(ViewRepresentation):
         near_sql = data_algebra.near_sql.NearSQLq(
             quoted_query_name=db_model.quote_identifier(view_name),
             prev_quoted_query_name=db_model.quote_identifier(prev_view_name),
-            query=data_algebra.near_sql.NearSQLContainer(sql_text=query),
+            query=query,
             terms=terms,
             temp_tables=temp_tables,
+            annotation='convert records'
         )
         return near_sql
 
