@@ -1,3 +1,4 @@
+
 import math
 import re
 import io
@@ -15,6 +16,43 @@ import data_algebra.eval_model
 import data_algebra.data_ops
 
 
+_have_sqlparse = False
+try:
+    # noinspection PyUnresolvedReferences
+    import sqlparse
+
+    _have_sqlparse = True
+except ImportError:
+    pass
+
+
+# noinspection PyBroadException
+def pretty_format_sql(sql, *, encoding=None, sqlparse_options=None):
+    assert isinstance(sql, str)
+    assert isinstance(encoding, (str, type(None)))
+    assert isinstance(encoding, (dict, type(None)))
+    if sqlparse_options is None:
+        sqlparse_options = {"reindent": True, "keyword_case": "upper"}
+    formatted_sql = sql
+    if _have_sqlparse:
+        try:
+            formatted_sql = sqlparse.format(
+                sql, encoding=encoding, **sqlparse_options
+            )
+        except Exception:
+            pass
+    return formatted_sql
+
+
+def _clean_annotation(annotation):
+    assert isinstance(annotation, (str, type(None)))
+    if annotation is None:
+        return annotation
+    annotation = annotation.strip()
+    annotation = re.sub(r'\s+', ' ', annotation)
+    return annotation
+
+
 # map from op-name to special SQL formatting code
 
 
@@ -113,6 +151,7 @@ def _db_is_in_expr(dbmodel, expression):
     )
 
 
+# noinspection PyUnusedLocal
 def _db_count_expr(dbmodel, expression):
     return 'SUM(1)'
 
@@ -120,7 +159,7 @@ def _db_count_expr(dbmodel, expression):
 def _db_concat_expr(dbmodel, expression):
     return (
         "("  # TODO: cast each to char on way in
-        + "|| ".join([dbmodel.expr_to_sql(ai, want_inline_parens=True) for ai in expression.args])
+        + " || ".join([dbmodel.expr_to_sql(ai, want_inline_parens=True) for ai in expression.args])
         + ")"
     )
 
@@ -197,6 +236,7 @@ class DBModel:
     drop_text: str
     string_type: str
     join_name_map: dict
+    supports_with: bool
 
     def __init__(
         self,
@@ -212,6 +252,7 @@ class DBModel:
         drop_text='DROP TABLE',
         string_type='VARCHAR',
         join_name_map=None,
+        supports_with=True
     ):
         if local_data_model is None:
             local_data_model = data_algebra.default_data_model
@@ -235,6 +276,7 @@ class DBModel:
         if join_name_map is None:
             join_name_map = {}
         self.join_name_map = join_name_map.copy()
+        self.supports_with = supports_with
 
     def db_handle(self, conn):
         return DBHandle(db_model=self, conn=conn)
@@ -324,7 +366,7 @@ class DBModel:
         r = r.reset_index(drop=True)
         return r
 
-    # noinspection PyMethodMayBeStatic
+    # noinspection PyMethodMayBeStatic,PyUnusedLocal
     def read_table(self, conn, table_name, *, qualifiers=None, limit=None):
         if not isinstance(table_name, str):
             raise TypeError("Expect table_name to be a str")
@@ -444,11 +486,10 @@ class DBModel:
         if len(missing) > 0:
             raise KeyError("referred to unknown columns: " + str(missing))
         cols_using = [c for c in table_def.column_names if c in using]
-        view_name = "table_reference_" + str(temp_id_source[0])
-        temp_id_source[0] = temp_id_source[0] + 1
+
         subsql = data_algebra.near_sql.NearSQLTable(
             terms={k: self.quote_identifier(k) for k in cols_using},
-            quoted_query_name=self.quote_identifier(view_name),
+            quoted_query_name=self.quote_table_name(table_def),
             quoted_table_name=self.quote_table_name(table_def),
         )
         near_sql = subsql
@@ -457,11 +498,12 @@ class DBModel:
             terms = OrderedDict()
             for k in using:
                 terms[k] = k
+            view_name = "table_reference_" + str(temp_id_source[0])
+            temp_id_source[0] = temp_id_source[0] + 1
             near_sql = data_algebra.near_sql.NearSQLUnaryStep(
-                previous_step_summary=subsql.summary(),
-                terms=terms,  # TODO: implement!
+                terms=terms,  # TODO: implement pruning
                 quoted_query_name=self.quote_identifier(view_name),
-                sub_sql=subsql.to_near_sql(columns=using),
+                sub_sql=subsql.to_bound_near_sql(columns=using),
                 temp_tables=subsql.temp_tables.copy(),
             )
         return near_sql
@@ -484,7 +526,7 @@ class DBModel:
                 subops[k] = op
         if len(subops) <= 0:
             # know using was not None is this case as len(extend_node.ops)>0 and all keys are in extend_node.column_set
-            return extend_node.sources[0].to_sql_implementation(
+            return extend_node.sources[0].to_near_sql_implementation(
                 db_model=self, using=using, temp_id_source=temp_id_source
             )
         if len(using) < 1:
@@ -494,7 +536,7 @@ class DBModel:
             raise KeyError("referred to unknown columns: " + str(missing))
         # get set of columns we need from subquery
         subusing = extend_node.columns_used_from_sources(using=using)[0]
-        subsql = extend_node.sources[0].to_sql_implementation(
+        subsql = extend_node.sources[0].to_near_sql_implementation(
             db_model=self, using=subusing, temp_id_source=temp_id_source
         )
         window_term = ""
@@ -524,11 +566,11 @@ class DBModel:
         view_name = "extend_" + str(temp_id_source[0])
         temp_id_source[0] = temp_id_source[0] + 1
         near_sql = data_algebra.near_sql.NearSQLUnaryStep(
-            previous_step_summary=subsql.summary(),
             terms=terms,
             quoted_query_name=self.quote_identifier(view_name),
-            sub_sql=subsql.to_near_sql(columns=subusing),
+            sub_sql=subsql.to_bound_near_sql(columns=subusing),
             temp_tables=subsql.temp_tables.copy(),
+            annotation=extend_node.to_python_implementation(print_sources=False, indent=-1)
         )
         return near_sql
 
@@ -545,7 +587,7 @@ class DBModel:
         subusing = project_node.columns_used_from_sources(using=using)[0]
         terms = {ci: self.expr_to_sql(oi) for (ci, oi) in subops.items()}
         terms.update({g: None for g in project_node.group_by})
-        subsql = project_node.sources[0].to_sql_implementation(
+        subsql = project_node.sources[0].to_near_sql_implementation(
             db_model=self, using=subusing, temp_id_source=temp_id_source
         )
         view_name = "project_" + str(temp_id_source[0])
@@ -555,12 +597,12 @@ class DBModel:
             group_terms = [self.quote_identifier(c) for c in project_node.group_by]
             suffix = "GROUP BY " + ", ".join(group_terms)
         near_sql = data_algebra.near_sql.NearSQLUnaryStep(
-            previous_step_summary=subsql.summary(),
             terms=terms,
             quoted_query_name=self.quote_identifier(view_name),
-            sub_sql=subsql.to_near_sql(columns=subusing),
+            sub_sql=subsql.to_bound_near_sql(columns=subusing),
             suffix=suffix,
             temp_tables=subsql.temp_tables.copy(),
+            annotation=project_node.to_python_implementation(print_sources=False, indent=-1)
         )
         return near_sql
 
@@ -574,7 +616,7 @@ class DBModel:
         if using is None:
             using = select_rows_node.column_set
         subusing = select_rows_node.columns_used_from_sources(using=using)[0]
-        subsql = select_rows_node.sources[0].to_sql_implementation(
+        subsql = select_rows_node.sources[0].to_near_sql_implementation(
             db_model=self, using=subusing, temp_id_source=temp_id_source
         )
         view_name = "select_rows_" + str(temp_id_source[0])
@@ -582,12 +624,12 @@ class DBModel:
         terms = {ci: None for ci in using}
         suffix = " WHERE " + self.expr_to_sql(select_rows_node.expr)
         near_sql = data_algebra.near_sql.NearSQLUnaryStep(
-            previous_step_summary=subsql.summary(),
             terms=terms,
             quoted_query_name=self.quote_identifier(view_name),
-            sub_sql=subsql.to_near_sql(columns=subusing),
+            sub_sql=subsql.to_bound_near_sql(columns=subusing),
             suffix=suffix,
             temp_tables=subsql.temp_tables.copy(),
+            annotation=select_rows_node.to_python_implementation(print_sources=False, indent=-1)
         )
         return near_sql
 
@@ -606,7 +648,7 @@ class DBModel:
         subusing = [
             c for c in select_columns_node.column_selection if c in subusing
         ]  # fix order
-        subsql = select_columns_node.sources[0].to_sql_implementation(
+        subsql = select_columns_node.sources[0].to_near_sql_implementation(
             db_model=self, using=set(subusing), temp_id_source=temp_id_source
         )
         # order/limit columns
@@ -629,7 +671,7 @@ class DBModel:
         if using is None:
             using = drop_columns_node.column_set
         subusing = drop_columns_node.columns_used_from_sources(using=using)[0]
-        subsql = drop_columns_node.sources[0].to_sql_implementation(
+        subsql = drop_columns_node.sources[0].to_near_sql_implementation(
             db_model=self, using=subusing, temp_id_source=temp_id_source
         )
         # /limit columns
@@ -649,7 +691,7 @@ class DBModel:
             using = order_node.column_set
         subusing = order_node.columns_used_from_sources(using=using)[0]
         subusing = [c for c in order_node.column_names if c in subusing]  # fix order
-        subsql = order_node.sources[0].to_sql_implementation(
+        subsql = order_node.sources[0].to_near_sql_implementation(
             db_model=self, using=set(subusing), temp_id_source=temp_id_source
         )
         view_name = "order_rows_" + str(temp_id_source[0])
@@ -671,12 +713,12 @@ class DBModel:
         if order_node.limit is not None:
             suffix = suffix + " LIMIT " + order_node.limit.__repr__()
         near_sql = data_algebra.near_sql.NearSQLUnaryStep(
-            previous_step_summary=subsql.summary(),
             terms=terms,
             quoted_query_name=self.quote_identifier(view_name),
-            sub_sql=subsql.to_near_sql(columns=subusing),
+            sub_sql=subsql.to_bound_near_sql(columns=subusing),
             suffix=suffix,
             temp_tables=subsql.temp_tables.copy(),
+            annotation=order_node.to_python_implementation(print_sources=False, indent=-1)
         )
         return near_sql
 
@@ -690,7 +732,7 @@ class DBModel:
         if using is None:
             using = rename_node.column_set
         subusing = rename_node.columns_used_from_sources(using=using)[0]
-        subsql = rename_node.sources[0].to_sql_implementation(
+        subsql = rename_node.sources[0].to_near_sql_implementation(
             db_model=self, using=subusing, temp_id_source=temp_id_source
         )
         view_name = "rename_" + str(temp_id_source[0])
@@ -704,11 +746,11 @@ class DBModel:
         }
         terms.update({vi: None for vi in unchanged_columns})
         near_sql = data_algebra.near_sql.NearSQLUnaryStep(
-            previous_step_summary=subsql.summary(),
             terms=terms,
             quoted_query_name=self.quote_identifier(view_name),
-            sub_sql=subsql.to_near_sql(columns=subusing),
+            sub_sql=subsql.to_bound_near_sql(columns=subusing),
             temp_tables=subsql.temp_tables.copy(),
+            annotation=rename_node.to_python_implementation(print_sources=False, indent=-1)
         )
         return near_sql
 
@@ -730,11 +772,11 @@ class DBModel:
         subusing = join_node.columns_used_from_sources(using=using.union(by_set))
         using_left = subusing[0]
         using_right = subusing[1]
-        sql_left = join_node.sources[0].to_sql_implementation(
+        sql_left = join_node.sources[0].to_near_sql_implementation(
             db_model=self, using=using_left, temp_id_source=temp_id_source
         )
         sub_view_name_left = sql_left.quoted_query_name
-        sql_right = join_node.sources[1].to_sql_implementation(
+        sql_right = join_node.sources[1].to_near_sql_implementation(
             db_model=self, using=using_right, temp_id_source=temp_id_source
         )
         sub_view_name_right = sql_right.quoted_query_name
@@ -784,19 +826,18 @@ class DBModel:
         temp_tables.update(sql_right.temp_tables)
         jointype = join_node.jointype
         try:
-            jointype = self.join_name_map[jointype]
+            jointype = self.join_name_map[jointype]  # TODO: maybe move this mapping earlier
         except KeyError:
             pass
         near_sql = data_algebra.near_sql.NearSQLBinaryStep(
             terms=terms,
             quoted_query_name=self.quote_identifier(view_name),
-            sub_sql1=sql_left.to_near_sql(columns=using_left),
-            previous_step_summary1=sql_left.summary(),
-            joiner=join_node.jointype + " JOIN",
-            sub_sql2=sql_right.to_near_sql(columns=using_right),
-            previous_step_summary2=sql_right.summary(),
+            sub_sql1=sql_left.to_bound_near_sql(columns=using_left),
+            joiner=jointype + " JOIN",
+            sub_sql2=sql_right.to_bound_near_sql(columns=using_right),
             suffix=on_terms,
             temp_tables=temp_tables,
+            annotation=join_node.to_python_implementation(print_sources=False, indent=-1)
         )
         return near_sql
 
@@ -819,10 +860,10 @@ class DBModel:
         using_right = subusing[1]
         if set(using_left) != set(using_right):
             raise ValueError("left/right usings did not match")
-        sql_left = concat_node.sources[0].to_sql_implementation(
+        sql_left = concat_node.sources[0].to_near_sql_implementation(
             db_model=self, using=using_left, temp_id_source=temp_id_source
         )
-        sql_right = concat_node.sources[1].to_sql_implementation(
+        sql_right = concat_node.sources[1].to_near_sql_implementation(
             db_model=self, using=using_left, temp_id_source=temp_id_source
         )
         view_name = "concat_rows_" + str(temp_id_source[0])
@@ -845,25 +886,71 @@ class DBModel:
             raise ValueError("name collisions on temp_tables: " + str(confused_temps))
         temp_tables = sql_left.temp_tables.copy()
         temp_tables.update(sql_right.temp_tables)
-        near_sql = data_algebra.near_sql.NearSQLUStep(
+        near_sql = data_algebra.near_sql.NearSQLBinaryStep(
             terms=terms,
             quoted_query_name=self.quote_identifier(view_name),
-            sub_sql1=sql_left.to_near_sql(
+            sub_sql1=sql_left.to_bound_near_sql(
                 columns=using_left,
                 force_sql=True,
                 constants=constants_left,
             ),
-            previous_step_summary1=sql_left.summary(),
-            sub_sql2=sql_right.to_near_sql(
+            joiner="UNION ALL",
+            sub_sql2=sql_right.to_bound_near_sql(
                 columns=using_right,
                 force_sql=True,
                 constants=constants_right,
             ),
-            previous_step_summary2=sql_right.summary(),
             temp_tables=temp_tables,
+            annotation=concat_node.to_python_implementation(print_sources=False, indent=-1)
         )
         return near_sql
 
+    def to_sql(
+        self,
+        ops,
+        *,
+        pretty=False,
+        annotate=False,
+        encoding=None,
+        sqlparse_options=None,
+        temp_tables=None,
+        use_with=False
+    ):
+        assert isinstance(self, DBModel)
+        assert isinstance(ops, data_algebra.data_ops.ViewRepresentation)
+        if sqlparse_options is None:
+            sqlparse_options = {"reindent": True, "keyword_case": "upper"}
+        ops.columns_used()  # for table consistency check/raise
+        temp_id_source = [0]
+        near_sql = ops.to_near_sql_implementation(
+            db_model=self, using=None, temp_id_source=temp_id_source
+        )
+        assert isinstance(near_sql, data_algebra.near_sql.NearSQL)
+        if (near_sql.temp_tables is not None) and (len(near_sql.temp_tables) > 0):
+            if temp_tables is None:
+                raise ValueError(
+                    "need temp_tables to be a dictionary to copy back found temporary table values"
+                )
+            temp_tables.update(near_sql.temp_tables)
+        sql_str = None
+        if use_with and self.supports_with:
+            sequence = near_sql.to_with_form()
+            len_sequence = len(sequence)
+            if len(sequence) >= 2:
+                sql_sequence = [None] * (len_sequence - 1)
+                for i in range(len_sequence - 1):
+                    nmi = sequence[i][0]  # already quoted
+                    sqli = sequence[i][1].to_sql(db_model=self, annotate=annotate)
+                    sql_sequence[i] = f' {nmi} AS (\n {sqli} \n)'
+                sql_last = sequence[len_sequence - 1].to_sql(db_model=self, force_sql=True, annotate=annotate)
+                sql_str = 'WITH\n' + ',\n'.join(sql_sequence) + '\n' + sql_last
+        if sql_str is None:
+            # non-with path
+            sql_str = near_sql.to_sql(db_model=self, force_sql=True, annotate=annotate)
+        if pretty:
+            sql_str = pretty_format_sql(sql_str, encoding=encoding, sqlparse_options=sqlparse_options)
+        return sql_str
+
     def row_recs_to_blocks_query(
         self, source_sql, record_spec, control_view, *, using=None, temp_id_source=None
     ):
@@ -931,7 +1018,7 @@ class DBModel:
             + source_sql
             + " ) a\n"
             + "CROSS JOIN (\n  "
-            + control_view.to_sql_implementation(
+            + control_view.to_near_sql_implementation(
                 self, using=using, temp_id_source=temp_id_source
             ).to_sql(db_model=self, columns=using, force_sql=True)
             + " ) b\n"
@@ -1000,6 +1087,135 @@ class DBModel:
         )
         return sql
 
+    # encode and name a term for use in a SQL expression
+    def enc_term_(self, k, *, terms):
+        v = None
+        try:
+            v = terms[k]
+        except KeyError:
+            pass
+        if v is None:
+            return self.quote_identifier(k)
+        return v + " AS " + self.quote_identifier(k)
+
+    def convert_nearsql_container_subsql_(self, nearsql_container, *, annotate=False):
+        assert isinstance(nearsql_container, data_algebra.near_sql.NearSQLContainer)
+        if isinstance(nearsql_container.near_sql, data_algebra.near_sql.NearSQLTable):
+            sql = (
+                    " "
+                    + nearsql_container.to_sql(self, annotate=annotate)
+                    + " "
+            )
+            if nearsql_container.near_sql.quoted_query_name != nearsql_container.near_sql.quoted_table_name:
+                sql = sql + (
+                        nearsql_container.near_sql.quoted_query_name
+                        + " "
+                )
+        elif isinstance(nearsql_container.near_sql, data_algebra.near_sql.NearSQLCommonTableExpression):
+            sql = (
+                    " "
+                    + nearsql_container.to_sql(self, annotate=annotate)
+                    + " "
+            )
+        else:
+            sql = (
+                    " ( "
+                    + nearsql_container.to_sql(self, annotate=annotate)
+                    + " ) "
+                    + nearsql_container.near_sql.quoted_query_name
+                    + " "
+            )
+        return sql
+
+    def nearsqltable_to_sql_(self, near_sql, *, columns=None, force_sql=False, constants=None, annotate=False):
+        assert isinstance(near_sql, data_algebra.near_sql.NearSQLTable)
+        if columns is None:
+            columns = [k for k in near_sql.terms.keys()]
+        if len(columns) <= 0:
+            force_sql = False
+        have_constants = (constants is not None) and (len(constants) > 0)
+        if force_sql or have_constants:
+            terms_strs = [self.quote_identifier(k) for k in columns]
+            if have_constants:
+                terms_strs = terms_strs + [
+                    v + " AS " + self.quote_identifier(k)
+                    for (k, v) in constants.items()
+                ]
+            if len(terms_strs) < 1:
+                terms_strs = [f'1 AS {self.quote_identifier("data_algebra_placeholder_col_name")}']
+            return "SELECT " + ", ".join(terms_strs) + " FROM " + near_sql.quoted_table_name
+        return near_sql.quoted_table_name
+
+    def nearsqlunary_to_sql_(self, near_sql, *, columns=None, force_sql=False, constants=None, annotate=False):
+        assert isinstance(near_sql, data_algebra.near_sql.NearSQLUnaryStep)
+        if columns is None:
+            columns = [k for k in near_sql.terms.keys()]
+        terms = near_sql.terms
+        if (constants is not None) and (len(constants) > 0):
+            terms.update(constants)
+        terms_strs = [self.enc_term_(k, terms=terms) for k in columns]
+        if len(terms_strs) < 1:
+            terms_strs = [f'1 AS {self.quote_identifier("data_algebra_placeholder_col_name")}']
+        sql = "SELECT "
+        if annotate and (near_sql.annotation is not None) and (len(near_sql.annotation) > 0):
+            sql = sql + " -- " + _clean_annotation(near_sql.annotation) + "\n "
+        sql = sql + ", ".join(terms_strs) + " FROM " + near_sql.sub_sql.convert_subsql(db_model=self, annotate=annotate)
+        if (near_sql.suffix is not None) and (len(near_sql.suffix) > 0):
+            sql = sql + " " + near_sql.suffix
+        return sql
+
+    def nearsqlbinary_to_sql_(self, near_sql, *, columns=None, force_sql=False, constants=None, annotate=False):
+        assert isinstance(near_sql, data_algebra.near_sql.NearSQLBinaryStep)
+        if columns is None:
+            columns = [k for k in near_sql.terms.keys()]
+        terms = near_sql.terms
+        if (constants is not None) and (len(constants) > 0):
+            terms.update(constants)
+        terms_strs = [self.enc_term_(k, terms=terms) for k in columns]
+        if len(terms_strs) < 1:
+            terms_strs = [f'1 AS {self.quote_identifier("data_algebra_placeholder_col_name")}']
+        sql = "SELECT "
+        if annotate and (near_sql.annotation is not None) and (len(near_sql.annotation) > 0):
+            sql = sql + " -- " + _clean_annotation(near_sql.annotation) + "\n "
+        sql = sql + (
+                ", ".join(terms_strs) + " FROM " + " ( "
+                + near_sql.sub_sql1.convert_subsql(db_model=self, annotate=annotate)
+                + " " + near_sql.joiner + " "
+                + near_sql.sub_sql2.convert_subsql(db_model=self, annotate=annotate)
+                )
+        if (near_sql.suffix is not None) and (len(near_sql.suffix) > 0):
+            sql = sql + " " + near_sql.suffix
+        sql = sql + " ) "
+        return sql
+
+    def nearsqlq_to_sql_(self, near_sql, *, columns=None, force_sql=False, constants=None, annotate=False):
+        assert isinstance(near_sql, data_algebra.near_sql.NearSQLq)
+        if columns is None:
+            columns = [k for k in near_sql.terms.keys()]
+        terms = near_sql.terms
+        if (constants is not None) and (len(constants) > 0):
+            terms.update(constants)
+
+        def enc_term(k):
+            v = terms[k]
+            if v is None:
+                return self.quote_identifier(k)
+            return v + " AS " + self.quote_identifier(k)
+
+        terms_strs = [enc_term(k) for k in columns]
+        if len(terms_strs) < 1:
+            terms_strs = [f'1 AS {self.quote_identifier("data_algebra_placeholder_col_name")}']
+        sql = "SELECT "
+        if annotate and (near_sql.annotation is not None) and (len(near_sql.annotation) > 0):
+            sql = sql + " -- " + _clean_annotation(near_sql.annotation) + "\n "
+        return sql + (
+            ", ".join(terms_strs)
+            + " FROM ( "
+            + near_sql.query
+            + " ) "
+            + near_sql.prev_quoted_query_name
+        )
+
     def __str__(self):
         return str(type(self).__name__)
 
@@ -1008,7 +1224,7 @@ class DBModel:
 
 
 class DBHandle(data_algebra.eval_model.EvalModel):
-    def __init__(self, *, db_model, conn, fns = None):
+    def __init__(self, *, db_model, conn, fns=None):
         if not isinstance(db_model, DBModel):
             raise TypeError(
                 "expected db_model to be of class data_algebra.db_model.DBHandle"
@@ -1023,6 +1239,7 @@ class DBHandle(data_algebra.eval_model.EvalModel):
     def __enter__(self):
         return self
 
+    # noinspection PyShadowingBuiltins
     def __exit__(self, type, value, traceback):
         self.close()
 
@@ -1036,9 +1253,9 @@ class DBHandle(data_algebra.eval_model.EvalModel):
     def describe_table(self, table_name, *, qualifiers=None, row_limit=7):
         head = self.read_query(
             q="SELECT * FROM "
-                + self.db_model.quote_table_name(table_name)
-                + " LIMIT "
-                + str(row_limit),
+              + self.db_model.quote_table_name(table_name)
+              + " LIMIT "
+              + str(row_limit),
         )
         return data_algebra.data_ops.describe_table(
             head,
@@ -1090,20 +1307,24 @@ class DBHandle(data_algebra.eval_model.EvalModel):
     def to_sql(self, ops,
                *,
                pretty=False,
+               annotate=False,
                encoding=None,
                sqlparse_options=None,
-               temp_tables=None):
+               temp_tables=None,
+               use_with=False):
         return ops.to_sql(self.db_model,
                           pretty=pretty,
+                          annotate=annotate,
                           encoding=encoding,
                           sqlparse_options=sqlparse_options,
-                          temp_tables=temp_tables)
+                          temp_tables=temp_tables,
+                          use_with=use_with)
 
     def drop_table(self, table_name):
         q_table_name = self.db_model.quote_table_name(table_name)
         self.execute(f'DROP TABLE IF EXISTS {q_table_name}')
 
-    def eval(self, ops, *, data_map=None, result_name=None, narrow=True):
+    def managed_eval(self, ops, *, data_map=None, result_name=None, narrow=True):
         query = ops.to_sql(self.db_model)
         if result_name is None:
             result_name = self.mk_tmp_name(data_map)
