@@ -10,7 +10,6 @@ import data_algebra.data_model
 import data_algebra.expr_rep
 import data_algebra.data_ops_types
 import data_algebra.connected_components
-import data_algebra.custom_functions
 
 
 # TODO: possibly import dask, Nvidia Rapids, or modin instead
@@ -24,12 +23,6 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         if not isinstance(pd, types.ModuleType):
             raise TypeError("Expected pd to be a module")
         self.pd = pd
-        self.custom_function_map = data_algebra.custom_functions.make_custom_function_map(
-            self
-        )
-        self.pandas_eval_env = {
-            k: cf.implementation for (k, cf) in self.custom_function_map.items()
-        }
 
     # utils
 
@@ -68,7 +61,7 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
     # bigger stuff
 
     # noinspection PyMethodMayBeStatic,PyUnusedLocal
-    def table_step(self, op, *, data_map, eval_env, narrow):
+    def table_step(self, op, *, data_map, narrow):
         if op.node_name != "TableDescription":
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.TableDescription"
@@ -88,7 +81,7 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         res = res.reset_index(drop=True)
         return res
 
-    def extend_step(self, op, *, data_map, eval_env, narrow):
+    def extend_step(self, op, *, data_map, narrow):
         if op.node_name != "ExtendNode":
             raise TypeError("op was supposed to be a data_algebra.data_ops.ExtendNode")
         window_situation = (
@@ -99,29 +92,15 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         if window_situation:
             op.check_extend_window_fns()
         res = op.sources[0].eval_implementation(
-            data_map=data_map, eval_env=eval_env, data_model=self, narrow=narrow
+            data_map=data_map, data_model=self, narrow=narrow
         )
-        standin_name = "_data_algebra_temp_g"  # name of an arbitrary input variable
+        data_algebra_temp_cols = {}
         if not window_situation:
             for (k, opk) in op.ops.items():
-                if isinstance(opk, data_algebra.user_fn.FnTerm):
-                    # res[k] = opk.value(*[res[nm.column_name] for nm in opk.args])
-                    pe = self.pandas_eval_env.copy()
-                    pe[opk.name] = opk.pandas_fn
-                    op_src = (
-                        "@"
-                        + opk.name
-                        + "("
-                        + ", ".join([nm.column_name for nm in opk.args])
-                        + ")"
-                    )
-                    res[k] = res.eval(op_src, local_dict=pe, global_dict=eval_env)
-                else:
-                    op_src = opk.to_pandas()
-                    res[k] = res.eval(
-                        op_src, local_dict=self.pandas_eval_env, global_dict=eval_env
-                    )
+                res_k = opk.evaluate(res)  # makes debugging easier
+                res[k] = res_k
         else:
+            standin_name = "_data_algebra_temp_g"  # name of an arbitrary input variable
             # build up a sub-frame to work on
             col_list = [c for c in set(op.partition_by)]
             col_set = set(col_list)
@@ -129,14 +108,26 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
                 if c not in col_set:
                     col_list = col_list + [c]
                     col_set.add(c)
-            order_cols = [c for c in col_list]  # must be partion by followed by order
+            order_cols = [c for c in col_list]  # must be partition by followed by order
+
             for (k, opk) in op.ops.items():
-                # assumes all args are column names, enforce this earlier
-                if len(opk.args) > 0:
-                    value_name = opk.args[0].to_pandas()
-                    if value_name not in col_set:
-                        col_list = col_list + [value_name]
-                        col_set.add(value_name)
+                # assumes all args are column names or values, enforce this earlier
+                if (len(opk.args) > 0):
+                    assert len(opk.args) == 1
+                    if isinstance(opk.args[0], data_algebra.expr_rep.ColumnReference):
+                        value_name = opk.args[0].column_name
+                        if value_name not in col_set:
+                            col_list = col_list + [value_name]
+                            col_set.add(value_name)
+                    elif isinstance(opk.args[0], data_algebra.expr_rep.Value):
+                        key = str(opk.args[0].value)
+                        if not key in data_algebra_temp_cols.keys():
+                            value_name = 'data_algebra_extend_temp_col_' + str(len(data_algebra_temp_cols))
+                            data_algebra_temp_cols[key] = value_name
+                            col_list.append(value_name)
+                            res[value_name] = opk.args[0].value
+                    else:
+                        raise ValueError("opk must be a ColumnReference or Value")
             ascending = [c not in set(op.reverse) for c in col_list]
             subframe = res[col_list].reset_index(drop=True)
             subframe["_data_algebra_orig_index"] = subframe.index
@@ -154,29 +145,38 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
             # perform calculations
             for (k, opk) in op.ops.items():
                 # work on a slice of the data frame
-                value_name = None
-                # assumes all args are column names, enforce this earlier
-                if len(opk.args) > 0:
-                    value_name = opk.args[0].to_pandas()
-                    if value_name not in set(col_list):
-                        col_list = col_list + [value_name]
                 # TODO: document exactly which of these are available
-                if len(opk.args) == 0:
-                    if opk.op == "row_number":
+                if len(opk.args) <= 0:
+                    if opk.op in ["row_number", "count"]:
                         subframe[k] = opframe.cumcount() + 1
                     elif opk.op == "ngroup":
                         subframe[k] = opframe.ngroup()
                     elif opk.op == "size":
                         subframe[k] = opframe[standin_name].transform(
                             opk.op
-                        )  # Pandas transform, not data_algegra
+                        )  # Pandas transform, not data_algebra
                     else:
                         raise KeyError("not implemented: " + str(k) + ": " + str(opk))
                 else:
                     # len(opk.args) == 1
-                    subframe[k] = opframe[value_name].transform(
-                        opk.op
-                    )  # Pandas transform, not data_algegra
+                    assert len(opk.args) == 1
+                    if isinstance(opk.args[0], data_algebra.expr_rep.ColumnReference):
+                        value_name = opk.args[0].column_name
+                        if value_name not in set(col_list):
+                            col_list = col_list + [value_name]
+                        subframe[k] = opframe[value_name].transform(
+                            opk.op
+                        )  # Pandas transform, not data_algegra
+                    elif isinstance(opk.args[0], data_algebra.expr_rep.Value):
+                        value_name = data_algebra_temp_cols[str(opk.args[0].value)]
+                        subframe[k] = opframe[value_name].transform(
+                            opk.op
+                        )  # Pandas transform, not data_algegra
+                    else:
+                        raise ValueError("opk must be a ColumnReference or Value")
+            # clear some temps
+            for value_name in data_algebra_temp_cols.values():
+                del res[value_name]
             # copy out results
             subframe = subframe.reset_index(drop=True)
             subframe = subframe.sort_values(by=["_data_algebra_orig_index"])
@@ -194,7 +194,7 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         # noinspection PyUnresolvedReferences
         return self.pd.DataFrame(cols)
 
-    def project_step(self, op, *, data_map, eval_env, narrow):
+    def project_step(self, op, *, data_map, narrow):
         if op.node_name != "ProjectNode":
             raise TypeError("op was supposed to be a data_algebra.data_ops.ProjectNode")
         # check these are forms we are prepared to work with, and build an aggregation dictionary
@@ -202,34 +202,48 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         # https://stackoverflow.com/questions/44635626/rename-result-columns-from-pandas-aggregation-futurewarning-using-a-dict-with
         # try the following tutorial:
         # https://www.shanelynn.ie/summarising-aggregation-and-grouping-data-in-python-pandas/
+        data_algebra_temp_cols = {}
+        res = op.sources[0].eval_implementation(
+            data_map=data_map, data_model=self, narrow=narrow
+        )
         for (k, opk) in op.ops.items():
             if len(opk.args) > 1:
                 raise ValueError(
                     "non-trivial aggregation expression: " + str(k) + ": " + str(opk)
                 )
             if len(opk.args) > 0:
-                if not isinstance(opk.args[0], data_algebra.expr_rep.ColumnReference):
+                if isinstance(opk.args[0], data_algebra.expr_rep.ColumnReference):
+                    pass
+                elif isinstance(opk.args[0], data_algebra.expr_rep.Value):
+                    key = str(opk.args[0].value)
+                    if not key in data_algebra_temp_cols.keys():
+                        value_name = 'data_algebra_project_temp_col_' + str(len(data_algebra_temp_cols))
+                        data_algebra_temp_cols[key] = value_name
+                        res[value_name] = opk.args[0].value
+                else:
                     raise ValueError(
-                        "windows expression argument must be a column: "
+                        "windows expression argument must be a column or value: "
                         + str(k)
                         + ": "
                         + str(opk)
                     )
-        res = op.sources[0].eval_implementation(
-            data_map=data_map, eval_env=eval_env, data_model=self, narrow=narrow
-        )
         res["_data_table_temp_col"] = 1
         if len(op.group_by) > 0:
             res = res.groupby(op.group_by)
         if len(op.ops) > 0:
             cols = {}
             for k, opk in op.ops.items():
+                value_name = None
+                if len(opk.args) > 0:
+                    value_name = str(opk.args[0])
+                    if isinstance(opk.args[0], data_algebra.expr_rep.Value):
+                        value_name = data_algebra_temp_cols[value_name]
                 if isinstance(opk, data_algebra.user_fn.FnTerm):
                     fn = opk.pandas_fn
-                    vk = res[str(opk.args[0])].agg(fn)
+                    vk = res[value_name].agg(fn)
                 else:
                     if len(opk.args) > 0:
-                        vk = res[str(opk.args[0])].agg(opk.op)
+                        vk = res[value_name].agg(opk.op)
                     else:
                         vk = res["_data_table_temp_col"].agg(opk.op)
                 cols[k] = vk
@@ -256,46 +270,46 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
             raise ValueError("result wasn't keyed by group_by columns")
         return res
 
-    def select_rows_step(self, op, *, data_map, eval_env, narrow):
+    def select_rows_step(self, op, *, data_map, narrow):
         if op.node_name != "SelectRowsNode":
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.SelectRowsNode"
             )
         res = op.sources[0].eval_implementation(
-            data_map=data_map, eval_env=eval_env, data_model=self, narrow=narrow
+            data_map=data_map, data_model=self, narrow=narrow
         )
-        q = op.expr.to_pandas()
-        res = res.query(q).reset_index(drop=True)
+        selection = op.expr.evaluate(res)
+        res = res.loc[selection, :].reset_index(drop=True, inplace=False)
         return res
 
-    def select_columns_step(self, op, *, data_map, eval_env, narrow):
+    def select_columns_step(self, op, *, data_map, narrow):
         if op.node_name != "SelectColumnsNode":
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.SelectColumnsNode"
             )
         res = op.sources[0].eval_implementation(
-            data_map=data_map, eval_env=eval_env, data_model=self, narrow=narrow
+            data_map=data_map, data_model=self, narrow=narrow
         )
         return res[op.column_selection]
 
-    def drop_columns_step(self, op, *, data_map, eval_env, narrow):
+    def drop_columns_step(self, op, *, data_map, narrow):
         if op.node_name != "DropColumnsNode":
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.DropColumnsNode"
             )
         res = op.sources[0].eval_implementation(
-            data_map=data_map, eval_env=eval_env, data_model=self, narrow=narrow
+            data_map=data_map, data_model=self, narrow=narrow
         )
         column_selection = [c for c in res.columns if c not in op.column_deletions]
         return res[column_selection]
 
-    def order_rows_step(self, op, *, data_map, eval_env, narrow):
+    def order_rows_step(self, op, *, data_map, narrow):
         if op.node_name != "OrderRowsNode":
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.OrderRowsNode"
             )
         res = op.sources[0].eval_implementation(
-            data_map=data_map, eval_env=eval_env, data_model=self, narrow=narrow
+            data_map=data_map, data_model=self, narrow=narrow
         )
         ascending = [
             False if ci in set(op.reverse) else True for ci in op.order_columns
@@ -305,13 +319,13 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         )
         return res
 
-    def rename_columns_step(self, op, *, data_map, eval_env, narrow):
+    def rename_columns_step(self, op, *, data_map, narrow):
         if op.node_name != "RenameColumnsNode":
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.RenameColumnsNode"
             )
         res = op.sources[0].eval_implementation(
-            data_map=data_map, eval_env=eval_env, data_model=self, narrow=narrow
+            data_map=data_map, data_model=self, narrow=narrow
         )
         return res.rename(columns=op.reverse_mapping)
 
@@ -330,16 +344,16 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
             pass
         return jointype
 
-    def natural_join_step(self, op, *, data_map, eval_env, narrow):
+    def natural_join_step(self, op, *, data_map, narrow):
         if op.node_name != "NaturalJoinNode":
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.NaturalJoinNode"
             )
         left = op.sources[0].eval_implementation(
-            data_map=data_map, eval_env=eval_env, data_model=self, narrow=narrow
+            data_map=data_map, data_model=self, narrow=narrow
         )
         right = op.sources[1].eval_implementation(
-            data_map=data_map, eval_env=eval_env, data_model=self, narrow=narrow
+            data_map=data_map, data_model=self, narrow=narrow
         )
         common_cols = set([c for c in left.columns]).intersection(
             [c for c in right.columns]
@@ -373,16 +387,16 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         res = res.reset_index(drop=True)
         return res
 
-    def concat_rows_step(self, op, *, data_map, eval_env, narrow):
+    def concat_rows_step(self, op, *, data_map, narrow):
         if op.node_name != "ConcatRowsNode":
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.ConcatRowsNode"
             )
         left = op.sources[0].eval_implementation(
-            data_map=data_map, eval_env=eval_env, data_model=self, narrow=narrow
+            data_map=data_map, data_model=self, narrow=narrow
         )
         right = op.sources[1].eval_implementation(
-            data_map=data_map, eval_env=eval_env, data_model=self, narrow=narrow
+            data_map=data_map, data_model=self, narrow=narrow
         )
         if op.id_column is not None:
             left[op.id_column] = op.a_name
@@ -392,12 +406,12 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         res = res.reset_index(drop=True)
         return res
 
-    def convert_records_step(self, op, *, data_map, eval_env, narrow):
+    def convert_records_step(self, op, *, data_map, narrow):
         if op.node_name != "ConvertRecordsNode":
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.ConvertRecordsNode"
             )
         res = op.sources[0].eval_implementation(
-            data_map=data_map, eval_env=eval_env, data_model=self, narrow=narrow
+            data_map=data_map, data_model=self, narrow=narrow
         )
         return op.record_map.transform(res, local_data_model=self)
